/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.weekday;

import org.tinygroup.weekday.config.WeekdayCheckStrategies;
import org.tinygroup.weekday.config.WeekdayCheckStrategy;
import org.tinygroup.weekday.exception.UnsureDateTimeException;

/**
 * 工作日检查
 * @author luoguo
 *
 */
public class WeekdayChecker implements DateTimeChecker {
	private WeekdayCheckStrategies weekdayCheckStrategies;

	public WeekdayCheckStrategies getWeekdayCheckStrategies() {
		return weekdayCheckStrategies;
	}

	public void setWeekdayCheckStrategies(
			WeekdayCheckStrategies weekdayCheckStrategies) {
		this.weekdayCheckStrategies = weekdayCheckStrategies;
	}

	public boolean isWorkingTime(PureTime pureTime) {
		for (WeekdayCheckStrategy weekdayCheckStrategy : weekdayCheckStrategies
				.getCheckStrategyList()) {
			TimeChecker timeChecker = weekdayCheckStrategy.getTimeChecker();
			if (timeChecker.isSuitableTime(pureTime)) {
				return timeChecker.isWorkingTime(pureTime);
			}
		}
		throw new UnsureDateTimeException();
	}

	public boolean isSuitableTime(PureTime pureTime) {
		for (WeekdayCheckStrategy weekdayCheckStrategy : weekdayCheckStrategies
				.getCheckStrategyList()) {
			TimeChecker timeChecker = weekdayCheckStrategy.getTimeChecker();
			if (timeChecker.isSuitableTime(pureTime)) {
				return true;
			}
		}
		throw new UnsureDateTimeException();
	}

	public boolean isWeekday(PureDate pureDate) {
		for (WeekdayCheckStrategy weekdayCheckStrategy : weekdayCheckStrategies
				.getCheckStrategyList()) {
			DateChecker dateChecker = weekdayCheckStrategy.getDateChecker();
			if (dateChecker.isSuitableDate(pureDate)) {
				return dateChecker.isWeekday(pureDate);
			}
		}
		throw new UnsureDateTimeException();
	}

	public boolean isSuitableDate(PureDate pureDate) {
		for (WeekdayCheckStrategy weekdayCheckStrategy : weekdayCheckStrategies
				.getCheckStrategyList()) {
			DateChecker dateChecker = weekdayCheckStrategy.getDateChecker();
			if (dateChecker.isSuitableDate(pureDate)) {
				return true;
			}
		}
		throw new UnsureDateTimeException();
	}

	public boolean isWorkingDateTime(PureDate pureDate, PureTime pureTime) {
		if (isSuitable(pureDate, pureTime)) {
			boolean isWeekday = isWeekday(pureDate);
			boolean isWorkingTime = isWorkingTime(pureTime);
			return isWeekday && isWorkingTime;
		}
		throw new UnsureDateTimeException();
	}

	public boolean isSuitable(PureDate pureDate, PureTime pureTime) {
		boolean isSuitableDate = isSuitableDate(pureDate);
		boolean isSuitableTime = isSuitableTime(pureTime);
		return isSuitableDate && isSuitableTime;
	}
}
