/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.weekday.config;

import org.tinygroup.weekday.DateChecker;
import org.tinygroup.weekday.DateTimeChecker;
import org.tinygroup.weekday.PureDate;
import org.tinygroup.weekday.PureTime;
import org.tinygroup.weekday.TimeChecker;

import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamAsAttribute;

/**
 * 检查
 * @author luoguo
 *
 */
@XStreamAlias("weekday-check-strategy")
public class WeekdayCheckStrategy implements DateTimeChecker {
	@XStreamAsAttribute
	private String name;
	private String description;
	private DateChecker dateChecker;
	private TimeChecker timeChecker;

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public DateChecker getDateChecker() {
		return dateChecker;
	}

	public void setDateChecker(DateChecker dateChecker) {
		this.dateChecker = dateChecker;
	}

	public TimeChecker getTimeChecker() {
		return timeChecker;
	}

	public void setTimeChecker(TimeChecker timeChecker) {
		this.timeChecker = timeChecker;
	}

	public boolean isWorkingDateTime(PureDate pureDate, PureTime pureTime) {
		if (dateChecker != null) {
			boolean dateCheck = dateChecker.isWeekday(pureDate);
			if (!dateCheck) {
				return false;
			}
		}
		if (timeChecker != null) {
			boolean timeCheck = timeChecker.isWorkingTime(pureTime);
			if (!timeCheck) {
				return false;
			}
		}
		return true;
	}

	public boolean isSuitable(PureDate pureDate, PureTime pureTime) {
		if (dateChecker != null) {
			boolean dateCheck = dateChecker.isSuitableDate(pureDate);
			if (!dateCheck) {
				return false;
			}
		}
		if (timeChecker != null) {
			boolean timeCheck = timeChecker.isSuitableTime(pureTime);
			if (!timeCheck) {
				return false;
			}
		}
		return true;
	}

	public boolean isWorkingTime(PureTime pureTime) {
		return timeChecker.isWorkingTime(pureTime);
	}

	public boolean isSuitableTime(PureTime pureTime) {
		return timeChecker.isSuitableTime(pureTime);
	}

	public boolean isWeekday(PureDate pureDate) {
		return dateChecker.isWeekday(pureDate);
	}

	public boolean isSuitableDate(PureDate pureDate) {
		return dateChecker.isSuitableDate(pureDate);
	}
}
